<?php

/**
 * 
 * @author Christian Doebler <christian.doebler@netways.de>
 *
 */
class IcingaApiSearchIdo
	extends IcingaApiSearch {

	/*
	 * VARIABLES
	 */

	protected $tablePrefix = 'icinga_';

	protected $queryMap = array (
		self::TARGET_HOST =>
			'select
				distinct ${FIELDS}
			from
				${TABLE_PREFIX}objects oh
			inner join ${TABLE_PREFIX}hosts h on h.host_object_id = oh.object_id
			inner join ${TABLE_PREFIX}hoststatus hs on hs.host_object_id = oh.object_id
			inner join ${TABLE_PREFIX}instances i on i.instance_id = h.instance_id
			inner join ${TABLE_PREFIX}host_contactgroups hcg on hcg.host_id = h.host_id
			inner join ${TABLE_PREFIX}contactgroups cg on cg.contactgroup_object_id = hcg.contactgroup_object_id
			inner join ${TABLE_PREFIX}objects ocg on ocg.object_id = hcg.contactgroup_object_id
			inner join ${TABLE_PREFIX}contactgroup_members cgm on cgm.contactgroup_id = cg.contactgroup_id
			inner join ${TABLE_PREFIX}objects oc on oc.object_id = cgm.contact_object_id
			where
				oh.objecttype_id = 1
				and ocg.objecttype_id = 11
			${FILTER}
			${GROUPBY}
			${ORDERBY}
			${LIMIT}',
		self::TARGET_SERVICE =>
			'select
				distinct ${FIELDS}
			from
				${TABLE_PREFIX}objects os
			inner join ${TABLE_PREFIX}services s on s.service_object_id = os.object_id
			inner join ${TABLE_PREFIX}instances i on i.instance_id = s.instance_id
			inner join ${TABLE_PREFIX}service_contactgroups scg on scg.service_id = s.service_id
			inner join ${TABLE_PREFIX}contactgroups cg on cg.contactgroup_object_id = scg.contactgroup_object_id
			inner join ${TABLE_PREFIX}contactgroup_members cgm on cgm.contactgroup_id = cg.contactgroup_id
			inner join ${TABLE_PREFIX}objects oc on oc.object_id = cgm.contact_object_id
			inner join ${TABLE_PREFIX}servicestatus ss on ss.service_object_id = os.object_id
			inner join ${TABLE_PREFIX}objects ocg on ocg.object_id = scg.contactgroup_object_id
			inner join ${TABLE_PREFIX}hoststatus hs on hs.host_object_id = s.host_object_id
			inner join ${TABLE_PREFIX}objects oh on oh.object_id = s.host_object_id
			where
				oh.objecttype_id = 1
				and os.objecttype_id = 2
				and ocg.objecttype_id = 11
			${FILTER}
			${GROUPBY}
			${ORDERBY}
			${LIMIT}',
		self::TARGET_HOSTGROUP =>
			'select
				distinct ${FIELDS}
			from
				${TABLE_PREFIX}objects ohg
			inner join ${TABLE_PREFIX}hostgroups hg on hg.hostgroup_object_id = ohg.object_id
			inner join ${TABLE_PREFIX}hostgroup_members hgm on hg.hostgroup_id = hgm.hostgroup_id
			inner join ${TABLE_PREFIX}objects oh on oh.object_id = hgm.host_object_id
			where
				ohg.objecttype_id=3
				and oh.objecttype_id=1
			${FILTER}
			${GROUPBY}
			${ORDERBY}
			${LIMIT}',
		self::TARGET_SERVICEGROUP =>
			'select
				distinct ${FIELDS}
			from
				${TABLE_PREFIX}objects osg
			inner join ${TABLE_PREFIX}servicegroups sg on sg.servicegroup_object_id = osg.object_id
			inner join ${TABLE_PREFIX}servicegroup_members sgm on sg.servicegroup_id = sgm.servicegroup_id
			inner join ${TABLE_PREFIX}objects os on os.object_id = sgm.service_object_id
			where
				osg.objecttype_id=4
				and os.objecttype_id=2
			${FILTER}
			${GROUPBY}
			${ORDERBY}
			${LIMIT}',
		self::TARGET_CONTACTGROUP =>
			'select
				distinct ${FIELDS}
			from
				 ${TABLE_PREFIX}objects ocg
			inner join ${TABLE_PREFIX}contactgroups cg on cg.contactgroup_object_id = ocg.object_id
			inner join ${TABLE_PREFIX}contactgroup_members cgm on cgm.contactgroup_id = cg.contactgroup_id
			inner join ${TABLE_PREFIX}objects oc on oc.object_id = cgm.contact_object_id
			where
				ocg.objecttype_id = 11
				and oc.objecttype_id = 10
				${FILTER}
				${GROUPBY}
				${ORDERBY}
				${LIMIT}',
                self::TARGET_TIMEPERIOD =>
                        'select
                                distinct ${FIELDS}
                        from
                                 ${TABLE_PREFIX}objects ot
			inner join ${TABLE_PREFIX}timeperiods t on t.timeperiod_object_id = ot.object_id
			inner join ${TABLE_PREFIX}timeperiod_timeranges tt on tt.timeperiod_id = t.timeperiod_id
			where
				ot.objecttype_id = 9
                                ${FILTER}
                                ${GROUPBY}
                                ${ORDERBY}
                                ${LIMIT}',
		self::TARGET_CUSTOMVARIABLE =>
			'select
				distinct ${FIELDS}
			from 
				${TABLE_PREFIX}customvariables cv
			inner join ${TABLE_PREFIX}customvariablestatus cvs on cv.object_id = cvs.object_id
                                ${FILTER}
                                ${GROUPBY}
                                ${ORDERBY}
                                ${LIMIT}',
		self::TARGET_CONFIG =>
			'select
				${FIELDS}
			from
				${TABLE_PREFIX}configfilevariables cfv
			where 1
				${FILTER}
				${GROUPBY}
				${ORDERBY}
				${LIMIT}',
		self::TARGET_PROGRAM =>
			'select
				${FIELDS}
			from
				distinct ${TABLE_PREFIX}processevents pe
			where
				event_type = 100
				${FILTER}
			order by
				event_time desc
			limit 1',
	);

	/*
	 * METHODS
	 */

	/**
	 * class constructor
	 *
	 * @param	void
	 * @return	void
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function __construct () {
		parent::__construct();
		return $this;
	}

	/**
	 * sets the table prefix
	 *
	 * @param	string		$prefix				table prefix
	 * @return	void
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function setTablePrefix ($prefix = null) {
		$this->tablePrefix = $prefix;
	}

	/**
	 * creates where statement for database query
	 *
	 * @param	void 
	 * @return	array							associative array w/ where statement and corresponding values
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function createQueryFilter () {

		$whereStatementArray = array();
		$whereStatementValues = array();

		foreach ($this->searchFilter as $key => $filterData) {

			$tmpStatementArray = array();

			foreach ($filterData as $matchType => $valueArray) {

				// create template for current part of where statement
				$separator = $key . ' ' . $matchType . ' ?';
				$numValues = count($valueArray);
				for ($x = 0; $x < $numValues; $x++) {
					array_push($tmpStatementArray, $separator);
				}

				// populate value array
				$whereStatementValues = array_merge($whereStatementValues, $valueArray);

				// create temporary statement
				$tmpStatement = '(' . implode(' or ', $tmpStatementArray) . ')';

			}

			// extend final where statement
			array_push($whereStatementArray, $tmpStatement);

		}

		// create final array
		$whereStatement = 'and ' . implode(' and ', $whereStatementArray);
		$returnData = array (
			'statement'	=> $whereStatement,
			'values'	=> $whereStatementValues,
		);

		return $returnData;

	}

	/**
	 * applies a search type to search columns
	 * @param	string				$fields			fields to extend by search type
	 * @return	string								modified fields
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	private function applySearchType ($fields) {
		switch ($this->searchType) {
			case IcingaApi::SEARCH_TYPE_COUNT:
				$fieldsArray = explode(' ', $fields);
				$countFieldsArray = count($fieldsArray);
				if ($countFieldsArray > 2) {
					throw new IcingaApiSearchException('applySearchType(): invalid number of columns for use of \'count\'!');
				} else {
					$suffix = 'count';
					if ($countFieldsArray == 2) {
						$suffix .= '_' . $fieldsArray[1];
					}
					$fields = 'count(' . $fieldsArray[0] . ') ' . $suffix;
				}
				break;
		}
		return $fields;
	}

	/**
	 * creates and executes database query
	 *
	 * @param	void
	 * @return	boolean							true on success otherwise false
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function executeQuery () {

		$success = true;

		if ($this->searchValid()) {

			// create base for query
			$query = $this->queryMap[$this->searchTarget];
			$fields = $this->applySearchType(implode(',', $this->resultColumns));
			$query = str_replace('${FIELDS}', $fields, $query);
			$query = str_replace('${TABLE_PREFIX}', $this->tablePrefix, $query);

			// add filter
			if (count($this->searchFilter)) {
				$filterData = $this->createQueryFilter();
				$filterStatement = $filterData['statement'];
				$queryValues = $filterData['values'];
			} else {
				$filterStatement = null;
				$queryValues = array();
			}
			$query = str_replace('${FILTER}', $filterStatement, $query);

			// add 'group by'
			$groupBy = (count($this->searchGroup)) ? (' group by ' . implode(',', $this->searchGroup) . ' ') : null;
			$query = str_replace('${GROUPBY}', $groupBy, $query);

			// add order
			$orderBy = (count($this->searchOrder)) ? (' order by ' . implode(',', $this->searchOrder) . ' ') : null;
			$query = str_replace('${ORDERBY}', $orderBy, $query);

			// add limit
			$limit = ($this->searchLimit !== false) ? (' limit ' . $this->searchLimit . ' ') : null;
			$query = str_replace('${LIMIT}', $limit, $query);

			$dbResult = $this->connectionObject->query($query, $queryValues);

		} else {

			$success = false;

		}

		return $success;

	}

	/**
	 * (non-PHPdoc)
	 * @see objects/search/IcingaApiSearch#fetch()
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function fetch () {
		$class = 'IcingaApiResultIdo';
		$object = new $class;
		if ($this->executeQuery()) {
			$object->setSearchObject($this->connectionObject->connectionStatement);
			if ($this->resultType !== false) {
				$object->setResultType($this->resultType);
			}
			$object->next();
		}
		parent::fetch();
		return $object;
	}

}

// extend exceptions
class IcingaApiSearchIdoException extends IcingaApiSearchException {}

?>
