<?php

/**
 * 
 * @author Christian Doebler <christian.doebler@netways.de>
 *
 */
class IcingaApiSearchIdoAbstraction
	extends IcingaApiSearchIdo {

	/*
	 * METHODS
	 */

	/**
	 * class constructor
	 *
	 * @param	void
	 * @return	void
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function __construct () {}

	/**
	 * fetches date information
	 *
	 * @param	void
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getProgramDate () {
		$object = $this->setSearchTarget(IcingaApi::TARGET_PROGRAM)
						->setResultColumns('PROGRAM_DATE')
						->fetch();
		return $object;
	}

	/**
	 * fetches version information
	 *
	 * @param	void
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getProgramVersion () {
		$object = $this->setSearchTarget(IcingaApi::TARGET_PROGRAM)
						->setResultColumns('PROGRAM_VERSION')
						->fetch();
		return $object;
	}

	/**
	 * fetches a list of contact groups
	 *
	 * @param	string				$contact			contact's name to use as filter (optional)
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getContactgroupsByContact ($contact = false) {
		$this->setSearchTarget(IcingaApi::TARGET_CONTACTGROUP)
				->setResultColumns('CONTACTGROUP_NAME');
		if ($contact !== false) {
			$this->setSearchFilter('CONTACT_NAME', $contact);
		}
		$object = $this->fetch();
		return $object;
	}

	/**
	 * fetches a list of hosts and corresponding status data
	 *
	 * @param	mixed				$contactgroups		contactgroup or array of contactgroups
	 * @param	string				$hostName			host name to use as filter (optional)
	 * @param	string				$hostState		host state to use as filter (optional)
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getHostStatusByContactgroup ($contactgroups, $hostName = false, $hostState = false) {
		$resultColumns = array(
			'HOST_NAME',
			'HOST_ALIAS',
			'HOST_DISPLAY_NAME',
			'HOST_ADDRESS',
			'HOST_OUTPUT',
			'HOST_PERFDATA',
			'HOST_CURRENT_STATE',
			'HOST_CURRENT_CHECK_ATTEMPT',
			'HOST_MAX_CHECK_ATTEMPTS',
			'HOST_LAST_CHECK',
			'HOST_LAST_STATE_CHANGE',
			'HOST_CHECK_TYPE',
			'HOST_LATENCY',
			'HOST_EXECUTION_TIME',
			'HOST_NEXT_CHECK',
			'HOST_HAS_BEEN_CHECKED',
			'HOST_LAST_HARD_STATE_CHANGE',
			'HOST_LAST_NOTIFICATION',
			'HOST_STATE_TYPE',
			'HOST_IS_FLAPPING',
			'HOST_PROBLEM_HAS_BEEN_ACKNOWLEDGED',
			'HOST_SCHEDULED_DOWNTIME_DEPTH',
			'HOST_STATUS_UPDATE_TIME'
		);
		$this->setSearchTarget(IcingaApi::TARGET_HOST)
				->setResultColumns($resultColumns)
				->setSearchFilter('CONTACTGROUP_NAME', $contactgroups)
				->setSearchGroup('HOST_ID')
				->setSearchOrder('HOST_NAME');
		if ($hostName !== false) {
			$this->setSearchFilter('HOST_NAME', $hostName);
		}
		if ($hostState !== false) {
			$this->setSearchFilter('HOST_CURRENT_STATE', $hostState);
		}
		$object = $this->fetch();
		return $object;
	}

	/**
	 * fetches a list of services and corresponding status data
	 *
	 * @param	string				$contactgroups		contactgroup or array of contactgroups
	 * @param	string				$hostName			host name to use as filter (optional)
	 * @param	string				$serviceName		service name to use as filter (optional)
	 * @param	string				$serviceState		service state to use as filter (optional)
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getServiceStatusByContactgroup ($contactgroups, $hostName = false, $serviceName = false, $serviceState = false) {
		$resultColumns = array(
			'HOST_STATUS_ALL',
			'SERVICE_NAME',
			'SERVICE_DISPLAY_NAME',
			'SERVICE_NOTIFICATIONS_ENABLED',
			'SERVICE_OUTPUT',
			'SERVICE_PERFDATA',
			'SERVICE_CURRENT_STATE',
			'SERVICE_CURRENT_CHECK_ATTEMPT',
			'SERVICE_MAX_CHECK_ATTEMPTS',
			'SERVICE_LAST_CHECK',
			'SERVICE_LAST_STATE_CHANGE',
			'SERVICE_CHECK_TYPE',
			'SERVICE_LATENCY',
			'SERVICE_EXECUTION_TIME',
			'SERVICE_NEXT_CHECK',
			'SERVICE_HAS_BEEN_CHECKED',
			'SERVICE_LAST_HARD_STATE_CHANGE',
			'SERVICE_LAST_NOTIFICATION',
			'SERVICE_STATE_TYPE',
			'SERVICE_IS_FLAPPING',
			'SERVICE_PROBLEM_HAS_BEEN_ACKNOWLEDGED',
			'SERVICE_SCHEDULED_DOWNTIME_DEPTH',
			'SERVICE_STATUS_UPDATE_TIME'
		);
		$this->setSearchTarget(IcingaApi::TARGET_SERVICE)
				->setResultColumns($resultColumns)
				->setSearchFilter('CONTACTGROUP_NAME', $contactgroups)
				->setSearchGroup('SERVICE_ID')
				->setSearchOrder(array('HOST_NAME', 'SERVICE_NAME'));
		if ($hostName !== false) {
			$this->setSearchFilter('HOST_NAME', $hostName);
		}
		if ($serviceName !== false) {
			$this->setSearchFilter('SERVICE_NAME', $serviceName);
		}
		if ($serviceState !== false) {
			$this->setSearchFilter('SERVICE_CURRENT_STATE', $serviceState);
		}
		$object = $this->fetch();
		return $object;
	}

	/**
	 * fetches configuration variables and assigned values
	 *
	 * @param	string				$key				configuration key to use as filter (optional)
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getConfig ($key = false) {
		$this->setSearchTarget(IcingaApi::TARGET_CONFIG)
			->setResultColumns(array('CONFIG_VAR_NAME', 'CONFIG_VAR_VALUE'))
			->setSearchOrder('CONFIG_VAR_NAME');
		if ($key !== false) {
			$this->setSearchFilter('CONFIG_VAR_NAME', $key, IcingaApi::MATCH_LIKE);
		}
		$object = $this->fetch();
		return $object;
	}

	/**
	 * fetches host execution times (min, average, max)
	 *
	 * @param	string				$contactgroups		contactgroup or array of contactgroups
	 * @param	string				$hostName			host name to use as filter (optional)
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getHostExecutionTimes ($contactgroups, $hostName = false) {
		$this->setSearchTarget(IcingaApi::TARGET_HOST)
				->setResultColumns(array('HOST_EXECUTION_TIME_MIN', 'HOST_EXECUTION_TIME_AVG', 'HOST_EXECUTION_TIME_MAX'))
				->setSearchFilter('CONTACTGROUP_NAME', $contactgroups)
				->setSearchGroup('HOST_CONFIG_TYPE');
		if ($hostName !== false) {
			$this->setSearchFilter('HOST_NAME', $hostName);
		}
		$object = $this->fetch();
		return $object;
	}

	/**
	 * fetches host latency times (min, average, max)
	 *
	 * @param	string				$contactgroups		contactgroup or array of contactgroups
	 * @param	string				$hostName			host name to use as filter (optional)
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getHostLatencyTimes ($contactgroups, $hostName = false) {
		$this->setSearchTarget(IcingaApi::TARGET_HOST)
				->setResultColumns(array('HOST_LATENCY_MIN', 'HOST_LATENCY_AVG', 'HOST_LATENCY_MAX'))
				->setSearchFilter('CONTACTGROUP_NAME', $contactgroups)
				->setSearchGroup('HOST_CONFIG_TYPE');
		if ($hostName !== false) {
			$this->setSearchFilter('HOST_NAME', $hostName);
		}
		$object = $this->fetch();
		return $object;
	}

	/**
	 * fetches service execution times (min, average, max)
	 *
	 * @param	string				$contactgroups		contactgroup or array of contactgroups
	 * @param	string				$hostName			host name to use as filter (optional)
	 * @param	string				$serviceName		service name to use as filter (optional)
	 * @param	string				$serviceState		service state to use as filter (optional)
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getServiceExecutionTimes ($contactgroups, $hostName = false, $serviceName = false) {
		$this->setSearchTarget(IcingaApi::TARGET_SERVICE)
				->setResultColumns(array('SERVICE_EXECUTION_TIME_MIN', 'SERVICE_EXECUTION_TIME_AVG', 'SERVICE_EXECUTION_TIME_MAX'))
				->setSearchFilter('CONTACTGROUP_NAME', $contactgroups)
				->setSearchGroup('SERVICE_CONFIG_TYPE');
		if ($hostName !== false) {
			$this->setSearchFilter('HOST_NAME', $hostName);
		}
		if ($serviceName !== false) {
			$this->setSearchFilter('SERVICE_NAME', $serviceName);
		}
		$object = $this->fetch();
		return $object;
	}

	/**
	 * fetches service latency times (min, average, max)
	 *
	 * @param	string				$contactgroups		contactgroup or array of contactgroups
	 * @param	string				$hostName			host name to use as filter (optional)
	 * @param	string				$serviceName		service name to use as filter (optional)
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getServiceLatencyTimes ($contactgroups, $hostName = false, $serviceName = false) {
		$this->setSearchTarget(IcingaApi::TARGET_SERVICE)
				->setResultColumns(array('SERVICE_LATENCY_MIN', 'SERVICE_LATENCY_AVG', 'SERVICE_LATENCY_MAX'))
				->setSearchFilter('CONTACTGROUP_NAME', $contactgroups)
				->setSearchGroup('SERVICE_CONFIG_TYPE');
		if ($hostName !== false) {
			$this->setSearchFilter('HOST_NAME', $hostName);
		}
		if ($serviceName !== false) {
			$this->setSearchFilter('SERVICE_NAME', $serviceName);
		}
		$object = $this->fetch();
		return $object;
	}

	/**
	 * fetches active hosts
	 *
	 * @param	string				$contactgroups		contactgroup or array of contactgroups
	 * @param	string				$hostName			host name to use as filter (optional; LIKE stamement)
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getActiveHostCount ($contactgroups, $hostName = false) {
		$this->setSearchTarget(IcingaApi::TARGET_HOST)
				->setSearchType(IcingaApi::SEARCH_TYPE_COUNT)
				->setResultColumns('HOST_OBJECT_ID')
				->setSearchFilter('CONTACTGROUP_NAME', $contactgroups)
				->setSearchFilter('HOST_ACTIVE_CHECKS_ENABLED', 1);
		if ($hostName !== false) {
			$this->setSearchFilter('HOST_NAME', $hostName);
		}
		$object = $this->fetch();
		return $object;
	}

	/**
	 * fetches number of passive hosts
	 *
	 * @param	string				$contactgroups		contactgroup or array of contactgroups
	 * @param	string				$hostName			host name to use as filter (optional; LIKE stamement)
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getPassiveHostCount ($contactgroups, $hostName = false) {
		$this->setSearchTarget(IcingaApi::TARGET_HOST)
				->setSearchType(IcingaApi::SEARCH_TYPE_COUNT)
				->setResultColumns('HOST_OBJECT_ID')
				->setSearchFilter('CONTACTGROUP_NAME', $contactgroups)
				->setSearchFilter('HOST_ACTIVE_CHECKS_ENABLED', 0);
		if ($hostName !== false) {
			$this->setSearchFilter('HOST_NAME', $hostName);
		}
		$object = $this->fetch();
		return $object;
	}

	/**
	 * fetches and returns an objects containing a list of hosts
	 *
	 * @param	void
	 * @return	IcingaApiResult							result object
	 * @author	Christian Doebler <christian.doebler@netways.de>
	 */
	public function getHostList () {
		$object = $this->setSearchTarget(IcingaApi::TARGET_HOST)
						->setResultColumns('HOST_NAME')
						->fetch();
		return $object;
	}

}

?>
